unit LinkIL_Apollo;

interface

uses
  Classes,
  Db,
  LinkIL,
  APwin,
  ApConn,
  ApoEnv,
  ApoQSet;

type
  TConnectionType = (ctLocal, ctRemote);

  TILLink_Apollo = class(TILLink)
  protected
    FConnectionType: TConnectionType;
    FConnection: TApolloConnection;
  public
    procedure DoClose; override;
    procedure DoOpen; override;
    function ExecSQL(const ASQL: string; const ARows: integer): integer; override;
    procedure FillTableList(ATableList: TStrings); override;
    function Generator(const AName: string; const ACount: integer = 1): integer; override;
    procedure GetTableList(ATableList: TStrings); override;
    procedure GetViewList(AViewList: TStrings); override;
    procedure GetStoredProcList(AStoredProcList: TStrings); override;
    function Query(const ASQL: string; const AOpen: Boolean = True;
     const AParams: TStrings = nil): TDataset; override;
    procedure TransBegin; override;
    procedure TransEnd; override;
    procedure TransRollback; override;
  end;

implementation

uses
  ApCommon, SDE60, SysUtils, ILConfig_Apollo, ILRegister;

{ TILLink_Apollo }

procedure TILLink_Apollo.DoClose;
begin
  if FConnectionType = ctRemote then begin
    FreeAndNil(FConnection);
  end;
end;

procedure TILLink_Apollo.DoOpen;
var
  LAccess,
  LServer: string;
  LIndex,
  LPort: Integer;

begin
  // FOptions is a string that contains the information for the connection
  // in the format amServer|Host|Port
  // If it is local then it only contains amLocal
  // DataSource is the database name (if local the folder, if remote the alias)
  // If it is remote, we create a TApolloConnection, set the props and set the
  // private FConnectionType to ctRemote so that later on we know how to
  // retrieve both the tablenames and open a query
  LIndex := AnsiPos('|', FOptions);
  if LIndex = 0 then begin
    LAccess := 'amLocal';
    LServer := '';
    LPort := -1;
  end else begin
    LAccess := Copy(FOptions, 1, LIndex - 1);
    Delete(FOptions, 1, LIndex);
    LIndex := AnsiPos('|', FOptions);
    LServer := Copy(FOptions, 1, LIndex - 1);
    Delete(FOptions, 1, LIndex);
    LPort := StrToIntDef(FOptions, -1);
  end;
  // end of parsing of FOptions
  if AnsiSameText(LAccess, 'amLocal') then begin
    FConnectionType := ctLocal;
  end else begin
    FConnectionType := ctRemote;
    FConnection := TApolloConnection.Create(self);
    FConnection.DatabaseName := FDataSource;
    FConnection.Host := LServer;
    FConnection.Port := LPort;
    FConnection.User := FUsername;
    FConnection.Password := FPassword;
    FConnection.Connect;
    if FConnection.Connected then begin
      FConnection.Login;
    end;
  end;
// Causing AV
//   sx_SysProp( 65025, Pointer(1) );
end;

function TILLink_Apollo.ExecSQL(const ASQL: string;
  const ARows: integer): integer;
begin
  with TApolloQuery(Query(ASQL, False)) do try
    ExecSQL;
    Result := -1; // TODO: Doesn't have this property of RowsAffected
    if (ARows <> -1) and (result <> ARows) then begin
    	raise Exception.Create('Row count mismatch.');
    end;
  finally free; end;
end;

function TILLink_Apollo.Generator(const AName: string;
  const ACount: integer): integer;
begin
  Result := -1;
end;

procedure TILLink_Apollo.GetStoredProcList(AStoredProcList: TStrings);
begin
  inherited;

end;

procedure TILLink_Apollo.FillTableList(ATableList: TStrings);
var
  LPointer: Pointer;
  LSearchRec: TSearchRec;
  LFound: Integer;
  LFilename: string;
begin
  // if it is remote, get the table names from the server
  if FConnectionType = ctRemote then begin
    LPointer := FConnection.GetTableNames;
    if LPointer <> nil then begin
      ATableList.LoadFromStream(TMemoryStream(LPointer));
    end;
  end else begin // find files in directory to list table names
    LFound := SysUtils.FindFirst(IncludeTrailingBackslash(DataSource) + '*.dbf' ,faAnyFile,LSearchRec);
    While LFound = 0 do begin
      LFileName := LSearchRec.Name;
      ATableList.Add(LFileName);
      LFound := SysUtils.FindNext(LSearchRec);
    end; { while }
    SysUtils.FindClose(LSearchRec);
  end;
end;

procedure TILLink_Apollo.GetTableList(ATableList: TStrings);
var
  LIndex: Integer;
begin
  FillTableList(ATableList);
  // Strip extensions from table names
  for LIndex := 0 to ATableList.Count - 1 do begin
    ATableList[LIndex] := Copy(ATableList[LIndex], 1, Pos('.', ATableList[LIndex]) - 1);
  end;
end;

procedure TILLink_Apollo.GetViewList(AViewList: TStrings);
begin
  inherited;

end;

function TILLink_Apollo.Query(const ASQL: string; const AOpen: Boolean;
  const AParams: TStrings): TDataset;
begin
  Result := TApolloQuery.Create(nil);
  with TApolloQuery(Result) do begin
    // depending on type of connection, set the ApolloConnection or not
    if FConnectionType = ctRemote then begin
      AccessMethod := amServer;
      ApolloConnection := FConnection;
    end;
    DatabaseName := FDataSource;
    SQL.Text := ParamSQL(ASQL, AParams);
    if AOpen then begin
      Open;
    end;
  end;
end;

procedure TILLink_Apollo.TransBegin;
begin

  inherited;
end;

procedure TILLink_Apollo.TransEnd;
begin
  inherited;

end;

procedure TILLink_Apollo.TransRollback;
begin
  inherited;

end;

initialization
  TILLinkRegistration.RegisterLink('Apollo', TILLink_Apollo, TformILConfigApollo);

end.
